from javascript import require, On
import datetime
import asyncio
import math
import json
import csv
import structlog
import os
import sys
import time
import re
from datetime import date
from tinydb import TinyDB
import subprocess
playerDatabase = TinyDB("playerDatabase.json")
# new_item = {"name": "Book", "quantity": 5}
# playerDatabase.insert(new_item) 

global api_bot

filestruc = "/"

if os.name == 'nt':
    filestruc = "\\"
else:
    filestruc = "/"




class MinecraftBot:
    def __init__(self, config: dict, useReturn = False, discordWebhook = None, useDiscordForms = False):
        """Main bot run loop"""
        global logger
        self.logger = structlog.get_logger()
        logger = self.logger
        # [:2]
        self.nodeVersion, self.pipVersion, self.pythonVersion = self.__versionsCheck()
        if discordWebhook != None:
            from discord import SyncWebhook
            from discord import Embed
            self.Embed = Embed
            self.useDiscordForms = useDiscordForms
            self.webhook = SyncWebhook.from_url(f"{discordWebhook}")
            embedVar = Embed(title="Sucsesfully Connected To The Webhook!", description=f"**Great news! The bot has successfully connected to this channel's webhook. From now on, it will send all the logs and valuable data right here, keeping you informed about everything happening on the server.**\n\n **Versions:**\n* [**Node**](https://nodejs.org/)**:      {self.nodeVersion[:2]}**\n* [**Pip**](https://pypi.org/project/pip/)**:          {self.pipVersion}**\n* [**Python**](https://www.python.org/)**:  {self.pythonVersion}**\n\n **Links:**\n* [**GitHub**](https://github.com/SilkePilon/OpenDeliveryBot)\n* [**Report Bugs**](https://github.com/SilkePilon/OpenDeliveryBot/issues)\n* [**Web Interface**](https://github.com/SilkePilon/OpenDeliveryBot-react)", color=0x3498db)
            embedVar.timestamp = datetime.datetime.utcnow()
            embedVar.set_footer(text='\u200b',icon_url="https://github.com/SilkePilon/OpenDeliveryBot/blob/main/chestlogo.png?raw=true")
            if useDiscordForms == True:
                today = date.today()
                self.webhook.send(content=f"{today}", thread_name=f"{today}", username="OpenDeliveryBot", avatar_url="https://github.com/SilkePilon/OpenDeliveryBot/blob/main/chestlogo.png?raw=true", embed=embedVar)
            else:
                try:
                    self.webhook.send(content="", username="OpenDeliveryBot", avatar_url="https://github.com/SilkePilon/OpenDeliveryBot/blob/main/chestlogo.png?raw=true", embed=embedVar)
                except:
                    self.logger.error(f"Detected that you are using a Forms channel but 'useDiscordForms' is set to False. Please change 'useDiscordForms' to True or provide a webhook url for a text channel.")
        self.discordWebhook = discordWebhook
        self.mineflayer = require('mineflayer')
        self.pathfinder = require('mineflayer-pathfinder')
        self.goals = require('mineflayer-pathfinder').goals
        self.mineflayerViewer = require('prismarine-viewer').mineflayer
        self.Vec3 = require("vec3").Vec3
        self.armorManager = require("mineflayer-armor-manager")
        self.autoeat = require('mineflayer-auto-eat').plugin
        self.repl = require('repl')
        self.statemachine = require("mineflayer-statemachine")
        os.system(f'{sys.argv[0]} -m javascript update')
        self.config = config
        self.logedin = False
        self.useReturn = useReturn
        
    
        self.script_directory = os.path.dirname(os.path.abspath(sys.argv[0]))
        self.bot = self.__create_bot()
        self.msa_data = False
        
        global api_bot
        api_bot = self.bot
        
    
    
    
    
        
    def __loging(self, message, icon="🤖", error=False, info=False, warning=False, chat=False, imageUrl:str=""):
        
        if self.useReturn == True:
            self.logger.info(f"{message}")
        elif self.discordWebhook != None:
            color = 0x3498db
            if error == True:
                color = 0x992d22
            if info == True:
                color = 0x3498db
            if warning == True:
                color = 0xe67e22
            if chat == True:
                color = 0x2ecc71
            embed = self.Embed(title="", description=f"**{message}**", color=color)
            embed.timestamp = datetime.datetime.utcnow()
            if imageUrl != "":
                embed.set_thumbnail(url=imageUrl)
            try:
                embed.set_footer(text=f'{self.bot.username}',icon_url=f"https://mc-heads.net/avatar/{self.bot.username}/600.png")
            except:
                embed.set_footer(text='\u200b',icon_url="https://github.com/SilkePilon/OpenDeliveryBot/blob/main/chestlogo.png?raw=true")
            if self.useDiscordForms == True:
                today = date.today()
                self.webhook.send(content=f"{today}", thread=f"{today}", username="OpenDeliveryBot", avatar_url="https://github.com/SilkePilon/OpenDeliveryBot/blob/main/chestlogo.png?raw=true", embed=embed)
            else:
                try:
                    self.webhook.send(content=f"", username="OpenDeliveryBot", avatar_url="https://github.com/SilkePilon/OpenDeliveryBot/blob/main/chestlogo.png?raw=true", embed=embed)
                except:
                    self.logger.error(f"Detected that you are using a Forms channel but 'useDiscordForms' is set to False. Please change 'useDiscordForms' to True or provide a webhook url for a text channel.")
        
        self.logger.info(f"{message}")
    
            
    def __msa(self, *msa):
        self.msa_data = msa[0]
        self.logger.info(f"{msa[0]['user_code']} MSA Code")
        
    
    
        
    def __versionsCheck(self):
        # Node
        result = subprocess.run(["node", "--version"], 
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
                                text=True)
        node_version = result.stdout.strip()
        # Remove leading 'v'
        node_version = node_version[1:] if node_version.startswith('v') else node_version
        # Remove periods
        node_version = node_version.replace('.', '')
        if int(node_version[:2]) >= 18:
            self.logger.info(f"Detected Node version {node_version[:2]} witch is supported!")
        else:
            self.logger.warning(f"Detected node version {node_version[:2]} witch is NOT supported!\nThis may cause problems. Please update to node 18 or above!")
            time.sleep(7)
            
            
        # Pip
        result = subprocess.run(["pip", "--version"], 
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
                                text=True)
        pip_version = result.stdout.strip()
        # Remove
        match = re.search(r'pip (\d+(?:\.\d+)*).*python (\d+(?:\.\d+)*)', pip_version)
        if match:
            pip_version = match.group(1)
            python_version = match.group(2)
        self.logger.info(f"Detected Pip version {pip_version} witch is supported!")
        self.logger.info(f"Detected Python version {python_version} witch is supported!")
        return node_version, pip_version, python_version
        
            
    

    def __create_bot(self):
        self.__loging(f"Connecting to {self.config['server_ip']}", info=True, imageUrl=f"https://eu.mc-api.net/v3/server/favicon/{self.config['server_ip']}")
        if self.config['version'] == "auto":
            self.version = False
        else:
            self.version = str(self.config['version'])
        return self.mineflayer.createBot({
            'host': self.config['server_ip'],
            'port': self.config['server_port'],
            'username': self.config['bot_name'],
            'password': self.config['password'],
            'auth': self.config['auth'],
            'version': self.version,
            'hideErrors': True,
            'onMsaCode': self.__msa
        })
    
    
    
    
    

    def start(self):
        # @On(self.bot, "login")
        # def on_login(*args):
        
            # r = self.repl.start('> ')
            # r.context.bot = self.bot
            self.__loging(f'Logged in as {self.bot.username}', info=True, imageUrl=f"https://mc-heads.net/avatar/{self.bot.username}/600.png")
            self.logedin = True
            self.__start_viewer()
            self.__setup_events()
            self.__loging(f'Cordinates: {int(self.bot.entity.position.x)}, {int(self.bot.entity.position.y)}, {int(self.bot.entity.position.z)}', info=True)
            self.__load_plugins()
            # self.__auto_totem()
            self.__equip_armor()
            self.__log_players()
            
            # self.__get_items()
            # self.__go_to_location()
            # self.__deposit_items()
        
    
        
    def __load_plugins(self):
        self.mcData = require('minecraft-data')(self.bot.version)
        self.bot.loadPlugin(self.pathfinder.pathfinder)
        self.bot.loadPlugin(self.armorManager)
        self.bot.loadPlugin(self.autoeat)
        self.movements = self.pathfinder.Movements(self.bot, self.mcData)
        self.movements.canDig = False

        self.bot.pathfinder.setMovements(self.movements)
        self.windows = require('prismarine-windows')(self.bot.version)
        self.Item = require('prismarine-item')(self.bot.version)


    
    
    def __setup_events(self):
        
        @On(self.bot, "path_update")
        def path_update(_, r):
            path = [self.bot.entity.position.offset(0, 0.5, 0)]
            for node in r['path']:
                path.append({'x': node['x'], 'y': node['y'] + 0.5, 'z': node['z']})
            self.bot.viewer.drawLine('path', path, 	0x0000FF)
        
        @On(self.bot.viewer, "blockClicked")
        def on_block_clicked(_, block, face, button):
            try:
                if button != 2:
                    return  # only right click
                p = block.position.offset(0, 1, 0)
                self.bot.pathfinder.goto(self.pathfinder.goals.GoalNear(p.x, p.y, p.z, 1), timeout=60)
            except:
                self.__loging(f"Cant get to goal", error=True)
        
        
        
        @On(self.bot, "death")
        def death(*args):
            self.bot.end()
            
            self.__loging("Bot died... stopping bot!", warning=True)
        
        @On(self.bot, "kicked")
        def kicked(this, reason, *a):
            self.bot.end()
            
            self.__loging("Kicked from server... stopping bot!", warning=True)
            
        @On(self.bot, "autoeat_started")
        def autoeat_started(item, offhand, *a):
            self.__loging(f"Eating {item['name']} in {'offhand' if offhand else 'hand'}", info=True)
            
        @On(self.bot, "autoeat_finished")
        def autoeat_finished(item, offhand):
            self.__loging(f"Finished eating {item['name']} in {'offhand' if offhand else 'hand'}", info=True)
            
        @On(self.bot, "error")
        def error(_, error):
            self.__loging(error, error=True)
        
        @On(self.bot, "end")
        def create_new_bot(*a):
            self.bot = self.__create_bot()
            
        @On(self.bot, 'chat')
        def handleMsg(this, sender, message, *args):
            if sender and (sender != self.bot.username):
                self.__loging(f"💬 {sender}: {message}", chat=True)
            

    def __equip_armor(self):
        self.bot.armorManager.equipAll()
        
        
    # def __auto_totem(self):
    #     totemId = self.bot.registry.itemsByName['totem_of_undying'].id  # Get the correct id
    #     if 'totem_of_undying' in self.bot.registry.itemsByName:
    #         import time

    #         def equip_totem():
    #             totem = self.bot.inventory.find_inventory_item(totemId, None)
    #             if totem:
    #                 self.bot.equip(totem, 'off-hand')

    #         while True:
    #             equip_totem()
    #             time.sleep(0.05)

    def __start_viewer(self):
        self.mineflayerViewer(self.bot, {"port": self.config['viewer_port']})
        self.__loging("Viewer started on port %s" % self.config['viewer_port'], info=True)
    
    def __log_players(self):
        print(type(self.bot.players))
        # playerDatabase.insert_multiple(dict(self.bot.players))
            
    def __item_By_Name(self, items, name):
            item = None
            for i in range(len(items)):
                item = items[i]
                if item and item['name'] == name:
                    return item
                return None
            
    def __add_values_to_csv(self, data, bot):
        now = datetime.datetime.now()
        server_info = f"{data['server_ip']}:{data['server_port']}"
        start_x = str(bot.entity.position.x)
        start_y = str(bot.entity.position.y)
        start_z = str(bot.entity.position.z)
        
        delivered_item = data["items_name"]

        distance = math.sqrt((self.config['x_coord'] - start_x)**2 + (self.config['z_coord'] - start_z)**2)

        row = [now, server_info, start_x, start_y, start_z, self.config['x_coord'], self.config['y_coord'], self.config['z_coord'], distance, delivered_item]
        os.makedirs(os.path.dirname(f'{self.script_directory}{filestruc}logs{filestruc}analytics.csv'), exist_ok=True)
        with open(f'{self.script_directory}{filestruc}logs{filestruc}analytics.csv', 'a', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(row)

    def __go_to_location(self, x, y, z):
        self.pathfinder.setGoal(
            self.goals.GoalNear(x, y, z, 1),
            timeout=60
        )
    
    def __get_items(self):
        x = self.config["chest_coords"][0]
        y = self.config["chest_coords"][1]
        z = self.config["chest_coords"][2]
        locaton = self.bot.pathfinder.setGoal(self.pathfinder.goals.GoalNear(x, y, z, 1), timeout=60)
        
        chestToOpen = self.bot.findBlock({
            'matching': [self.mcData.blocksByName[name].id for name in [f'{str(self.config["chest_type"]).lower()}']],
            'maxDistance': 10,
        })
        
        if chestToOpen:

            chest = self.bot.openContainer(self.bot.blockAt(self.Vec3(-62, 72, 47)))
            
            item = self.__item_By_Name(chest.containerItems(), self.config["items_name"])
            
            if item:
                try:
                    self.windows.withdraw(item.type, None, self.config['items_count'])
                except Exception as err:
                    self.bot.chat(f"unable to withdraw {self.config['items_count']} {item.name}")
            else:
                self.bot.chat(f"unknown item {self.config['items_name']}")
            
            asyncio.sleep(5)
            
            chest.close()                
            
        else:
            self.bot.chat("Can't find the chest")

    def __deposit_items(self):

        foundChest = False
        
        while not foundChest:
            
            chestToOpen = self.bot.findBlock({
                'matching': [self.mcData.blocksByName[name].id for name in ['chest']], 
                'maxDistance': self.config["chest_range"],
            })
        
            if not chestToOpen and foundChest == False:
                self.__loging("No delivery chest found", error=True)
                break
                
            if chestToOpen.position.x:
                x = chestToOpen.position.x
                y = chestToOpen.position.y  
                z = chestToOpen.position.z

                locaton = self.bot.pathfinder.setGoal(self.pathfinder.goals.GoalNear(x, y, z, 1), timeout=60)
                
                try:
                    chest = self.bot.openContainer(chestToOpen)
                except Exception:
                    print(Exception)
                    continue
                
                item = next((item for item in chest.slots if item and item.name == f'{self.config["items_name"]}'), None)
                self.windows.deposit(item, "null", "null", "null")
                
                chest.close()

                foundChest = True
                
                break
            
        self.__add_values_to_csv(self.config, self.bot)
        
    # Needs to be changed to {"item": item_count}
    def inventory(self):
        inv = []
        if self.bot and self.bot.inventory:
            for item in self.bot.inventory.items():
                inv.append(item.displayName)
        return inv
        
    
    
    def coordinates(self):
        if self.logedin == True:
            return f"{int(self.bot.entity.position.x)}, {int(self.bot.entity.position.y)}, {int(self.bot.entity.position.z)}"
        
    def custom_code(self, code:str=""):
        """Run Custom Code on the bot"""
        if self.logedin == True:
            self.__loging("Running custom code on the bot is not reconmended!", warning=True)
            bot = self.bot
            response = eval(code)(bot)
            return response
    
    def stop(self):
        self.bot.end()
        self.__loging("Stopped bot!", warning=True)
        



import streamlit as st
from st_on_hover_tabs import on_hover_tabs
import streamlit_toggle as tog
from streamlit_extras.add_vertical_space import add_vertical_space
from streamlit_extras.stateful_button import button
from streamlit_extras.no_default_selectbox import selectbox
import streamlit.components.v1 as components
from streamlit_extras.grid import grid
from javascript import require, On
import pandas as pd
import numpy as np
from contextlib import redirect_stdout
import shlex
from streamlit.runtime.scriptrunner import get_script_run_ctx
from contextlib import contextmanager
from io import StringIO
import time
import math
import subprocess
import datetime
import requests
import json
import console_ui
import asyncio
import contextlib
from functools import wraps
from io import StringIO
# from bot import makeBot
from streamlit.runtime.scriptrunner import add_script_run_ctx
import os
import sys


import opendeliverybot.bot as OpenDeliveryBot


filestruc = "/"

if os.name == 'nt':
    filestruc = "\\"
else:
    filestruc = "/"


st.set_page_config(
    page_title="Bot controller",
    page_icon="🕹",
    layout="centered",
    initial_sidebar_state="auto",
)
script_directory = os.path.dirname(os.path.abspath(sys.argv[0]))
with open(f'{script_directory}{filestruc}data{filestruc}data.json', 'r') as f:
    data = json.load(f)

def webmode():
    def capture_output(func):
        """Capture output from running a function and write using streamlit."""

        @wraps(func)
        def wrapper(*args, **kwargs):
            # Redirect output to string buffers
            stdout, stderr = StringIO(), StringIO()
            try:
                with contextlib.redirect_stdout(stdout), contextlib.redirect_stderr(stderr):
                    return func(*args, **kwargs)
            except Exception as err:
                st.write(f"Failure while executing: {err}")
            finally:
                if _stdout := stdout.getvalue():
                    st.write("Execution stdout:")
                    st.code(_stdout)
                if _stderr := stderr.getvalue():
                    st.write("Execution stderr:")
                    st.code(_stderr)

        return wrapper
    stprint = capture_output(print)
    
    
    
    
    
    st.markdown("<style>" + open(f"{script_directory}{filestruc}frontend{filestruc}styles.css").read() + "</style>", unsafe_allow_html=True)

    with st.sidebar:
        tabs = on_hover_tabs(
            tabName=["Settings", "Dashboard", "Analytics"],
            iconName=["settings", "dashboard", "analytics"],
            styles={
                "navtab": {
                    "background-color": "#111",
                    "color": "#818181",
                    "font-size": "18px",
                    "transition": ".3s",
                    "white-space": "nowrap",
                    "text-transform": "uppercase",
                },
                "tabOptionsStyle": {
                    ":hover :hover": {"color": "white", "cursor": "pointer"}
                },
                "iconStyle": {
                    "position": "fixed",
                    "left": "7.5px",
                    "text-align": "left",
                },
                "tabStyle": {
                    "list-style-type": "none",
                    "margin-bottom": "30px",
                    "padding-left": "30px",
                },
            },
            key="1",
        )

    if tabs == "Dashboard":
        st.title("Open Delivery Bot")
            
        col1, col2, col3, col4, col5 = st.columns([2, 2, 2, 2, 2])

        with col1:
            x_coord = st.text_input("x", label_visibility="collapsed", placeholder="X coord")
                
        with col2:
            y_coord = st.text_input("y", label_visibility="collapsed", placeholder="Y coord")
            
        with col3:
            z_coord = st.text_input("z", label_visibility="collapsed", placeholder="Z coord")

        with col5:
            # def isViewerRunning():
            #     try:
            #         url = "http://127.0.0.1:1000"
            #         get_url = requests.get(url)
            #         code = get_url.status_code
                    
            #         if code == 200:
            #             return True
            #     except requests.exceptions.RequestException:
            #         return False
                
            

            # if isViewerRunning:
            #     start_dropoff_bot = st.button("Deliver item", disabled=True)
            # else:
            start_dropoff_bot = st.button("Deliver item")
            
        with col4:
            # if isViewerRunning:
            #     prismarineViewer_button = st.button("Open bot view", key="viewer_button", on_click=openViewer)
            # else:
            # prismarineViewer_button = st.button("Open bot view", key="viewer_button", on_click=openViewer)
            if start_dropoff_bot:
                try:
                    with open(f'{script_directory}{filestruc}data{filestruc}data.json', 'r') as file:
                        settings = json.load(file)
                        
                   
                    bot = OpenDeliveryBot.MinecraftBot(settings, streamlit=st)
                    
                    bot.start()
                    
                    
                    
                    st.toast("Bot started!", icon="✅")
                    st.button("Stop bot", on_click=bot.stop)
                except ValueError:
                    st.toast("Please enter a valid float number", icon="🚨")
        if start_dropoff_bot:
            Inv = st.empty()
            cord = st.empty()
            time.sleep(5)
            if bot.logedin == True:
                st.components.v1.iframe("http://127.0.0.1:1000", width=700, height=400, scrolling=False)
                while True:
                    Inv.text("Inventory: " + str(bot.inventory()))
                    cord.text("Coordinates: " + str(bot.coordinates()))
                
                    # screen.markdown('<iframe width="700rem" height="400rem" src="http://127.0.0.1:1000" frameborder="0" scrolling="auto" allowfullscreen id="render"></iframe>', unsafe_allow_html=True)
                    time.sleep(5)
            
            
            # render = st.markdown('<iframe width="700rem" height="400rem" src="http://127.0.0.1:1000" frameborder="0" scrolling="auto" allowfullscreen id="render"></iframe><br><br><br><br><br><br><br><br><br><br><br><br><br><br>', unsafe_allow_html=True)
            
        
    if tabs == "Settings":
        st.title("Bot settings")
        bot_name = st.text_input(label="Bot name", placeholder="BOT", value=data["bot_name"])

        col1, col2 = st.columns((2, 5))
        
        with col1:
            server_ip = st.text_input(label="Server ip", placeholder="127.0.0.1", value=data["server_ip"])
            viewer_ip = st.text_input(label="Viewer ip", placeholder="127.0.0.1", value=data["viewer_ip"])
            password = st.text_input(label="Password", type="password", value=data["password"])
            chest_type = selectbox("Select the chest type", ["chest", "trapped_chest"], no_selection_label=f"Selected: {data['chest_type']}")
                        
            def save_changes():
                with open(f'{script_directory}{filestruc}data{filestruc}data.json', 'r') as file:
                    data = json.load(file)
                
                data['server_ip'] = server_ip
                data['server_port'] = server_port
                data['bot_name'] = bot_name
                data['quit_on_low_health'] = healt_toggle
                data['low_health_threshold'] = range_slider
                data['password'] = password
                data['auth'] = auth
                data['viewer_ip'] = viewer_ip
                data['viewer_port'] = viewer_port
                data['chest_type'] = chest_type
                data['chest_range'] = chest_range
                data['version'] = version
                
                with open(f'{script_directory}{filestruc}data{filestruc}data.json', 'w') as file:
                    json.dump(data, file, indent=4)


        with col2:
            server_port = st.text_input(label="Server port", placeholder="25565", value=data["server_port"])
            viewer_port = st.text_input(label="Viewer port", placeholder="2000", value=data["viewer_port"])
            
            auth = selectbox("Select an authentication method", ["microsoft", "cracked"], no_selection_label=f"Selected: {data['auth']}")
            version = selectbox("Select an Minecraft version", ["auto", "1.19", "1.18", "1.17", "1.16", "1.15", "1.14", "1.13", "1.12"], no_selection_label=f"Selected: {data['version']}")
            chest_range = st.text_input(label="Chest range", placeholder="100", value=data["chest_range"])

        st.markdown("""---""")
        st.header("WIP section")
        add_vertical_space(1)
        
        wip_col1, wip_col2 = st.columns((2, 5))
        
        with wip_col1:
            healt_toggle = tog.st_toggle_switch(label="Quit on low health", 
                        key="Key1",
                        default_value=data["quit_on_low_health"], 
                        label_after = False, 
                        inactive_color = '#D3D3D3', 
                        active_color="#11567f", 
                        track_color="#29B5E8",
                        )
            
        with wip_col2:
            range_slider = st.slider(label=" ", label_visibility="collapsed" , min_value=1, max_value=19, value=data["low_health_threshold"], disabled=not healt_toggle)

        add_vertical_space(2)
        
        save_button = st.button(label="Save changes")
        
        if save_button:
            save_changes()
            st.toast("Saved changes!")
    
    if tabs == 'Analytics':
        st.title("Delivery analytics")
        st.header("WIP - Placeholder values only")
        csv_df = pd.DataFrame(np.random.randn(20, 3), columns=["a", "b", "c"])
        my_grid = grid(2, [2, 4, 1], 1, 4, vertical_align="bottom")
        
        my_grid.dataframe(csv_df, use_container_width=True)
        my_grid.line_chart(csv_df, use_container_width=True)

webmode()



import os
import sys
import click
from opendeliverybot.api import api

@click.command(context_settings={"ignore_unknown_options": True})
@click.option("--console", default=False, is_flag=True, help="Force the app to use the console")
@click.argument('args', nargs=-1)
def run(console, args):

    is_linux = sys.platform.startswith('linux')
    is_mac = sys.platform == 'darwin'
    is_windows = sys.platform in ['win32', 'cygwin']

    script_directory = os.path.dirname(os.path.abspath(__file__))

    if console:
        os.system(f"python {script_directory}/console_ui.py {' '.join(args)}")
    else:
        if is_linux or is_mac:
            if os.environ.get('DISPLAY') == '':
                print('No GUI, using console UI')
                os.system(f"python {script_directory}/console_ui.py {' '.join(args)}")
            else:
                print('Using GUI')
                # print('Web ui is still in developmet and is not working atm, rerun the command again with --console')
                api()
        elif is_windows:
            print('Using GUI')
            # print('Web ui is still in developmet and is not working atm, rerun the command again with --console')
            api()
        else:
            print('Unknown platform, using console UI')
            os.system(f"python {script_directory}/console_ui.py {' '.join(args)}")

if __name__ == "__main__":
    run()


from fastapi import FastAPI
import uvicorn
from opendeliverybot.bot import MinecraftBot
import time
from javascript import require
import json
from fastapi.responses import JSONResponse
app = FastAPI()


def api():
    @app.get("/api/v1/login")
    async def startup():
        try:
            account.stop()
        except: pass
        config = {
        "server_ip": "menu.mc-complex.com", 
        "server_port": 25565,
        "bot_name": "silke2007minecraft@gmail.com",
        "password": "",  
        "auth": "microsoft",
        "version": "1.12",
        "viewer_port": 1000,
        "chest_coords": [10, 64, -8],
        "chest_range": 64,
        "chest_type": "chest",
        "items_name": "cobblestone",
        "items_count": 64,
        "x_coord": 0,
        "y_coord": 70, 
        "z_coord": 0
        }

        bot = MinecraftBot(config)
        return JSONResponse(content={"message": "Bot started"})


    @app.get("/api/v1/get_msa")
    async def startup(email:str = ""):
        config = {
        "server_ip": "blockmania.com", 
        "server_port": 25565,
        "bot_name": email,
        "password": "",  
        "auth": "microsoft",
        "version": "1.12",
        "viewer_port": 1000,
        "chest_coords": [10, 64, -8],
        "chest_range": 64,
        "chest_type": "chest",
        "items_name": "cobblestone",
        "items_count": 64,
        "x_coord": 0,
        "y_coord": 70, 
        "z_coord": 0
        }
        global account
        account = MinecraftBot(config)
        msa_status = False
        loops = 0
        try:
            if account.bot.username != None:
                account.stop()
                JSONResponse(content={"msa": "Already logged in!"})
                msa_status = True
            while msa_status == False:
                if loops >= 10:
                    
                    return JSONResponse(content={"msa": f"Max tries exceeded!"})
                
                try:
                    if account.msa_data['user_code'] != False:
                        return JSONResponse(content={"msa": f"{account.msa_data['user_code']}"})
                except:
                    continue
                loops += 1
        except:
            account.stop()
            return JSONResponse(content={"msa": f"Error"})


    
    uvicorn.run(app)